#include "main.h"
#include "objects.h"

void play_sound(GLint sound_id)
{
	if(sound_channels[sound_id] < 0)
	{
		/* start playing walk sound */
		sound_channels[sound_id] = Mix_PlayChannel(-1, sound[sound_id], -1);
	}
}

void stop_sound(GLint sound_id)
{
	if(sound_channels[sound_id] > -1)
	{
		/* stop playing sound */
		Mix_HaltChannel(sound_channels[sound_id]);
		sound_channels[sound_id] = -1;
	}
}

void stop_all_sounds()
{
	/* Mix_HaltChannel(-1) will not work here,
	   because we also need sound_channels[sound_id] to be -1! */
	
	for(loop = 0; loop < NUM_SOUNDS; ++loop)
	{
		stop_sound(loop);
	}
}

void load_sound(GLbyte *name, GLint sound_id)
{
	GLbyte name2[strlen(name)];
	
	strcpy(name2, name);
		
	sound[sound_id] = Mix_LoadWAV(name2);
	
	if (!sound[sound_id])
	{
		/* quick hack, changes to ogg to wav. see docs/INSTALL.win32, why*/
		name2[strlen(name2)-3] = 'w';
		name2[strlen(name2)-2] = 'a';
		name2[strlen(name2)-1] = 'v';
		
		sound[sound_id] = Mix_LoadWAV(name2);
		
		if (!sound[sound_id])
		{
			fprintf(stderr, "%s: %s\n", name2,Mix_GetError()); /* Fehler beim laden: Meldung ausgeben und Programm beenden */
			quit (-1);
		}
	}
	
	sound_channels[sound_id] = -1; /* the channel is not played at the moment */
}

void load_all_sounds()
{
	load_sound(SOUND_PATH_walk, SOUND_walk);
	load_sound(SOUND_PATH_wind, SOUND_wind);
	load_sound(SOUND_PATH_menuchange, SOUND_menuchange);
	load_sound(SOUND_PATH_atom, SOUND_atom);
	load_sound(SOUND_PATH_explode, SOUND_explode);
	load_sound(SOUND_PATH_jump, SOUND_jump);
	
	music=Mix_LoadMUS(MUSIC_PATH_ingame);
	if(!music) {
		printf("Mix_LoadMUS(): %s\n", Mix_GetError());
		quit(-1);
	}

}

void create_level() /* create random level, w/o breaking any level rules */
{
	GLint temp;
	GLint acidcount = 0;
	
	/* There must be some ground under Atomio's feet at the beginning :D */
	for (loop=0; loop<14; ++loop)
	{
		lvl[loop][LEVEL_SIZE_y-1] = 'g';
	}
	
	for (loop=14; loop<LEVEL_SIZE_x-1; ++loop)
	{	
		temp = rand() % 2;
		
		/* If acidblocks are >4, Atomio won't be able to jump over them */
		if (temp == 1 && acidcount == 4)
		{
			temp = 0;
			acidcount = 0;
		}
		
		switch(temp)
		{
			case 0:
					lvl[loop][LEVEL_SIZE_y-1] = 'g';
					acidcount = 0;
					break;
			case 1:
					lvl[loop][LEVEL_SIZE_y-1] = 'a';
					++acidcount;
					break;
			default:
					break;
		}
		
		/* Acidblocks MUST NOT be <2 */
		if (temp == 1 && loop<LEVEL_SIZE_x && acidcount <2)
		{
			++loop;
			lvl[loop][LEVEL_SIZE_y-1] = 'a';
			++acidcount;
		}
	}
	
	level_created = GL_TRUE;
}
/*____________________________________________*/

void BuildFont(void) /* Schrift aus Textur erstellen */
{

    GLfloat cx;
    GLfloat cy;

    /* 256 Display Lists erstellen (fr jedes Zeichen 1) */
    font  = glGenLists( 256 );
    /* Schrifttextur auswhlen */
    glBindTexture( GL_TEXTURE_2D, texture[TEXTURE_font] );

    /* Alle 256 Listen durchgehen */
    for ( loop = 0; loop < 256; loop++ )
        {
	    /* x Position des Zeichens */
	    cx = 1 - ( float )( loop % 16 ) / 16.0f;
	    /* y Position des Zeichens */
	    cy = 1 - ( float )( loop / 16 ) / 16.0f;


	    glNewList( font + ( 255 - loop ), GL_COMPILE );
	      /* Fr jedes Zeichen ein Quadrat erstellen */
	      glBegin( GL_QUADS );
	      	glTexCoord2f( cx - 0.0625, cy );	glVertex2i( 0, 0 );
			glTexCoord2f( cx, cy );				glVertex2i( 16, 0 );
			glTexCoord2f( cx, cy - 0.0625f );	glVertex2i( 16, 16 );
			glTexCoord2f( cx - 0.0625f, cy - 0.0625f);	glVertex2i( 0, 16 );
	      glEnd( );

	      /* den "Cursor" nach rechts setzen, damit das nchste Zeichen nicht alles berschreibt */
	      glTranslated( 10, 0, 0 );
	    glEndList( );
        }
}

void glPrint( GLint x, GLint y, char *string, int set ) /* Gibt Text aus */
{
    if ( set > 1 )
	set = 1;

    /* Schrifttextur auswhlen */
    glBindTexture( GL_TEXTURE_2D, texture[TEXTURE_font] );

    /* Tiefentest ausschalten, damit der Text "auf" das Bild gezeichnet wird */
    glDisable( GL_DEPTH_TEST );
	//glDisable(GL_LIGHTING);
    glEnable(GL_BLEND);

    glMatrixMode( GL_PROJECTION );
    glPushMatrix( );

    glLoadIdentity( );
    /* Orthogonales Rendern aktivieren: es gibt keinen Fluchtpunkt mehr, alles ist 2D */
    glOrtho( 0, 640, 0, 480, -1, 1 );

    glMatrixMode( GL_MODELVIEW );
    glPushMatrix( );

    glLoadIdentity( );

    glTranslated( x, y, 0 ); /* "Cursor" positionieren */

    glListBase( font - 32 + ( 128 * set ) ); /* Schriftart auswhlen (0=normal, 1=kursiv) */

    glCallLists( strlen( string ), GL_BYTE, string ); /* Text schreiben */

    glMatrixMode( GL_PROJECTION );
    glPopMatrix( );

    glMatrixMode( GL_MODELVIEW );

    glPopMatrix( );

    /* Tiefentest wieder aktivieren */
    glEnable( GL_DEPTH_TEST );
	//glEnable(GL_LIGHTING);
    glDisable(GL_BLEND);
}

/*____________________________________________*/

void calcFrameRate(void)	/* Framerate errechnen, in der 1. Sekunde wird mll ausgegeben: egal, Sync klappt trotzdem */
{
	static float framesPerSecond = 0.0f;	/* Beinhaltet die aktuellen FPS */
	static float lastTime = 0.0f;		/* Speichert die Zeit, die fr den letzten Frame gebraucht wurde*/

	static float frameTime = 0.0f;			/* Speichert die Zeit, die fr den Frame gebraucht wurde, wichtig fr die Animationen! */
	float currentTime = SDL_GetTicks() * 0.001f;

	/* Speichert die vergangene Zeit zwischen dem jetzigen und dem letzten Frame,
	dann wird die aktuelle Zeit fr den nchsten Frame gespeichert */
	FrameInterval = currentTime - frameTime;
	frameTime = currentTime;

	/* Framecounter hochzhlen */
	++framesPerSecond;

	if( currentTime - lastTime > 1.0f )
	{
		lastTime = currentTime; /* Die alte Zeit auf die aktuelle setzen. Dient alt Startpunkt fr die nchste Sekunde */

		fps=framesPerSecond;

		framesPerSecond = 0; /* FPS zurcksetzen */
	}
}

void load_texture(GLbyte *name, GLint tex_id)	/* Bitmap als OpenGL Textur laden */
{
	SDL_Surface *TextureImage[1];	/* Platz fr die Textur freimachen */
	SDL_RWops *rwop;
	GLbyte name2[strlen(name)];
	
	strcpy(name2, name);
	
	rwop=SDL_RWFromFile(name2, "rb");

	if (!(TextureImage[0] = IMG_Load_RW(rwop,1))) /* Bild laden und auf Fehler berprfen */
  	{
		/* quick hack, changes to png to bmp. see docs/INSTALL.win32, why
		 the if-then-else below belongs to this hack also (swaps GL_RGB with GL_BGR) */
		name2[strlen(name2)-3] = 'b';
		name2[strlen(name2)-2] = 'm';
		name2[strlen(name2)-1] = 'p';
		
		rwop=SDL_RWFromFile(name2, "rb");
		
		if (!(TextureImage[0] = IMG_Load_RW(rwop,1))) /* try to load the bmp now */
		{
			fprintf(stderr, "%s: %s\n", name2, IMG_GetError()); /* Fehler beim laden: Meldung ausgeben und Programm beenden */
			quit (-1);
		}
	}

	/* everything allright, lets create our texture */
	glGenTextures( 1, &texture[tex_id] );
	glBindTexture( GL_TEXTURE_2D, texture[tex_id] );
	if(name2[strlen(name2)-3] == 'b') /* if bmp then swap colors */
	{
		glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage[0]->w,
				  TextureImage[0]->h, 0, GL_BGR,
				  GL_UNSIGNED_BYTE, TextureImage[0]->pixels );
	}
	else /* png */
	{
		glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage[0]->w,
				  TextureImage[0]->h, 0, GL_RGB,
				  GL_UNSIGNED_BYTE, TextureImage[0]->pixels );
	}
	
	/* Lineares Filtern anfordern */
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	
	if ( TextureImage[0] ) SDL_FreeSurface( TextureImage[0] ); /* Speicher wieder freigeben */
}

static void load_all_textures()	/* Alle Texturen einladen, ruft die Funktion load_texture() auf */
{
	load_texture(TEXTURE_PATH_font, TEXTURE_font);
	load_texture(TEXTURE_PATH_g, TEXTURE_g);
	load_texture(TEXTURE_PATH_g_top, TEXTURE_g_top);
	load_texture(TEXTURE_PATH_b, TEXTURE_b);
	load_texture(TEXTURE_PATH_skybox, TEXTURE_skybox);
	load_texture(TEXTURE_PATH_acid, TEXTURE_acid);
	load_texture(TEXTURE_PATH_acidfog, TEXTURE_acidfog);
	load_texture(TEXTURE_PATH_menu, TEXTURE_menu);
}

void setup_engine(GLint width, GLint height, GLboolean fullscreen)	/* Fenster erstellen, OpenGL initialisieren, usw. */
{
	const SDL_VideoInfo* info = NULL;	/* Informationen ber die aktuellen Video Einstellungen */

	/* values for the light */
	GLfloat LightAmbient[]= { 0.1f, 0.1f, 0.1f, 0.1f };
	GLfloat LightDiffuse[]= { 0.3f, 0.3f, 0.3f, 0.1f };
	GLfloat LightSpec[]={0.1f, 1.0f, 0.1f, 1.0f};
	GLfloat LightPosition[]= { 0.0f, 0.0f, 10.0f, 1.0f };
	GLfloat spotlightDirection[]={0.0f,0.0f,-1.0f,1.0f};
	GLfloat matSpecular[]={0.2f,0.2f,0.2f,0.1f};
	
	GLint bpp = 0;	/* Speichert die Farbtiefe */
	GLint flags = 0;	/* Speichert die Flags fr SDL_SetVideoMode */

	/* we setup sound first, because when the audio device is locked,
	   we won't be stuck in the fullscreen window (like in q3a...) */
	
	/* the audio setting we like to get */
	int audio_rate = 22050;
	Uint16 audio_format = AUDIO_S16; 
	int audio_channels = 2;
	int audio_buffers = 512;
	
	if (SDL_Init(SDL_INIT_AUDIO) < 0)
	{	
		/* Fehler: Meldung ausgeben und Programm beenden */
		fprintf( stderr, "Audio initialization failed: %s\n", SDL_GetError( ) );
		quit(1);
	}
	
	if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers))
	{
		printf("Mix_OpenAudio failed!\n");
		quit(1);
	}	
	
	if( SDL_Init( SDL_INIT_VIDEO ) < 0 )	/* SDL Video Subsystem initialisieren */
	{
		/* Fehler: Meldung ausgeben und Programm beenden */
		fprintf( stderr, "Video initialization failed: %s\n", SDL_GetError( ) );
		quit(1);
	}

	info = SDL_GetVideoInfo( );	/* Video Informationen sammeln */

	if( !info )
	{
		/* Fehler: Meldung ausgeben und Programm beenden (Sollte niemals vorkommen...)*/
		fprintf( stderr, "Video query failed: %s\n", SDL_GetError( ) );
		quit(1);
	}

	bpp = info->vfmt->BitsPerPixel;	/* Es wird die aktuelle Farbtiefe verwendet, da nicht alle Systeme diese im Betrieb ndern knnen. Immer schn Plattformunabhngig bleiben ;) */

	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 ); /* _Mindestens_ 5 Bit pro Frabkanal */
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 ); /* 16-Bit Z-Buffer */
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 ); /* Der Doublebuffer verhindert flackern! */

	flags = SDL_OPENGL;
	if (fullscreen)	/* wurde der #define FULLSCREEN auf GL_TRUE gesetzt? */
		flags += SDL_FULLSCREEN;

	if( SDL_SetVideoMode(width, height, bpp, flags ) == 0) /* Videomodus umschalten */
	{
		/* Fehler: Meldung ausgeben und Programm beenden
		Passiert wenn die Auflsung nicht verfgbar ist oder (unter UNIX) die DISPLAY Variable nicht gesetzt ist, usw*/
		fprintf(stderr, "Video mode set failed: %s\n", SDL_GetError( ));
		quit(1);
	}

	SDL_WM_SetCaption("Atom 'o' Run",""); /* Fenstertitel und Icon setzen */ /* 1. fenster-titel, 2. icon */
	SDL_ShowCursor(SDL_DISABLE); /* Mauscursor verstecken */

	/* Ab hier ist das Fenster erstellt worden */

	float ratio = (float) width / (float) height;


	glEnable(GL_TEXTURE_2D); /* Texture Mapping aktivieren */

	glShadeModel(GL_SMOOTH); /* Gouraud-Shading aktivieren, sorgt fr weiche Polygonbergnge */

	/* Culling. */
	glCullFace(GL_BACK);
	glFrontFace(GL_CCW);
	glEnable(GL_CULL_FACE);
	
	/* lighting */
	glLightfv(GL_LIGHT0, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, LightSpec);
	glLightfv(GL_LIGHT0, GL_POSITION,LightPosition);
	glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spotlightDirection);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
	glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 126.0f);
	glMaterialfv(GL_FRONT, GL_SPECULAR, matSpecular);
	
	//glEnable(GL_LIGHTING);
	
	
	/*fr die Transparenz*/
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glClearColor(0, 0, 0, 0); /* Farbe, mit der das Fenster gelscht wird */

	glClearDepth( 1.0f ); /* Optionen fr den Tiefenpuffer */
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);

	glViewport(0, 0, width, height); /* Viewport setzen, errechnet sich aus der Fenstergrsse */

	glMatrixMode(GL_PROJECTION); /* Projektionsmatrix setzen */
	glLoadIdentity();

	gluPerspective( 60.0, ratio, 1.0, 1024.0 ); /* blubb, soweit der einzigste GLU aufruf... */

	Mix_AllocateChannels(NUM_SOUNDS); // allocate NUM_SOUNDS mixing channels 
	load_all_sounds();

	load_all_textures();	/* Alle Texturen laden */
	BuildFont( ); /* Die Schrift erstellen */
	Build_dlists(); /* Objekte erstellen */
	
	srand(time(0)); /* Zufallsgenerator initialisieren */
}
