#include "main.h"
#include "events.h"
#include "engine_setup.h"
#include "scenes.h"
#include "objects.h"

GLint resx = 800;
GLint resy = 600;
GLboolean fullscreen = 1;

void quit(GLint code)
{	
	for(loop = 0; loop < NUM_SOUNDS; ++loop)
	{
		Mix_FreeChunk(sound[loop]);
	}
	
	Mix_FreeMusic(music);
	music=NULL;
	
	Mix_CloseAudio();
	
	glDeleteLists(font, 256+NUM_DLISTS); /* Schrift & Objekte lschen und Speicher freigeben */
	for(loop = 0; loop < NUM_TEXTURES; ++loop) /* Alle Texturen freigeben */
	{
		glDeleteTextures( NUM_TEXTURES, &texture[loop] );
	}
	
	SDL_ShowCursor(SDL_ENABLE); /* Mauscursor wieder sichtbar machen */
	
	SDL_Quit();

	/* Programm Beenden */
	exit(code);
}

void process_args(int argc, char** argv)
{
    GLbyte argstr[255]={0};

	/* see which arguments are given: */
    for(loop = 1; loop < argc; ++loop)
    {
		strcpy(argstr, argv[loop]);
				
        switch( *argstr )
        {
            case '/':
            case '-':
				switch(argstr[1])
				{
					case 'f':
						if (loop+1 != argc)
						{
							++loop;
							switch(argv[loop][0])
							{
								case '0':
									fullscreen = 0;
									break;
								case '1':
									fullscreen = 1;
									break;
								default:
									printf("Please select if you want fullscreen or not.\nSee \"atomorun -h\" for more help\n");
									quit(-1);
									break;
							}
						}
						else
						{
							printf("Please select a resolution.\nSee \"atomorun -h\" for more help\n");
							quit(-1);
						}
						break;
					case 'r':
						if (loop+1 != argc)
						{
							++loop;
							switch(argv[loop][0])
							{
								case '1':
									resx=320;
									resy=240;
									break;
								case '2':
									resx=640;
									resy=480;
									break;
								case '3':
									resx=800;
									resy=600;
									break;
								case '4':
									resx=1024;
									resy=768;
									break;
								case '5':
									resx=1280;
									resy=1024;
									break;
								case '6':
									resx=1600;
									resy=1200;
									break;
								default:
									printf("Please select a resolution.\nSee \"atomorun -h\" for more help\n");
									quit(-1);
								break;
							}
						}	
						else
						{
							printf("Please select a resolution.\nSee \"atomorun -h\" for more help\n");
							quit(-1);
						}						
						break;
					case 'v':
						printf("%s\n", VERSION);
						quit(0);
						break;
					case 'h':
						printf("Atomorun help:\n", VERSION);
						printf("--------------\n");
						printf("Usage: atomorun [OPTION]...\n\n");
						printf("  -r X\tSelect resolution. X must be one of the following:\n");
						printf("\t\t1 - 320x240 (very ugly...)\n");
						printf("\t\t2 - 640x480\n");
						printf("\t\t3 - 800x600 (default)\n");
						printf("\t\t4 - 1024x768\n");
						printf("\t\t5 - 1280x1024\n");
						printf("\t\t6 - 1600x1200\n");
						printf("  -f X\tSelect fullscreen mode. X must be one of the following:\n");
						printf("\t\t0 - No fullscreen\n");
						printf("\t\t1 - Fullscreen (default)\n");
						printf("  -v\tPrint Atom'o'run version and exit\n");
						printf("Report bugs to <atomorun@whosme.de>\n");
						printf("Visit http://atomorun.whosme.de\n");
						quit(0);
						break;
					
					default:
						printf("Unknown argument %s\nSee \"atomorun -h\" for more help\n", argstr);
						quit(-1);
						break;
				}
				break;
			default:
				printf("Unknown argument %s\nSee \"atomorun -h\" for more help\n", argstr);
				quit(-1);
                break;
		}
    }
}

int main(int argc, char** argv)
{	
	level_created = GL_FALSE;
	levelrun = GL_FALSE;
	
	process_args(argc, argv);
	
	set_default(); /* Standardwerte festlegen: */
	
	setup_engine(resx, resy, fullscreen); /* Fenster erstellen, OpenGL initialisieren, usw */
	
	while(1)
	{
		/*Das Hauptmen: */
		intro_running = GL_FALSE;
		menu();
			stop_all_sounds();
		if (levelrun) level();
	}
	
	/* Dieser Punkt wird NIEMALS erreicht werden */
	return 0;
}
