#include "main.h"
#include "events.h"
#include "objects.h"
#include "engine_setup.h"

/* Charakter- und Kollisionsabfrage- bedingte Variablen */
GLint Position, Position2, hoehe, hoehe2, sprunghoehe;
GLboolean jump, jumptranslating, moving, obenf, untenf, rechtsf, linksf, runter, lookleft;
GLfloat x, y, xs, yalt, xrots1, yrots2, xrots2, yrots2, xversatz1, yversatz1, bombe;

/* some variables we need for the intro */
GLfloat acidfogX; /* the x-position of the acid fog */
GLboolean bomb_exploded = GL_FALSE;
GLboolean intro_jump = GL_FALSE;

void set_default(void) /* Standardwerte festlegen: */
{
	camz = -15.9;
	camx = -12.0f;
	xrots2 = 90.0f;
	yrots2 = 220.0f;
	xversatz1 = -1.6f;
	yversatz1 = -1.6f;
	x=10;
	xs=sprunghoehe=2;
	jumptranslating=0;
	bombe=0.5f;
	hoehe=0;
	lookleft = GL_TRUE;
	
	acidfogX = -22.5f;
	intro_running = GL_TRUE;
	bomb_exploded = GL_FALSE;
	intro_jump = GL_FALSE;
	
	Mix_HaltMusic();
}

void char_dead(void)
{
	levelrun = GL_FALSE;
	stop_all_sounds();
	Mix_HaltMusic();
	
	play_sound(SOUND_wind);
	
	while (!scene_ende)
	{
		process_events();
		
		if (key_return)
		{
			scene_ende = !scene_ende;
			key_return = GL_FALSE; /* otherwise the menu would read this event, too */
		}
		
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); /* Farb- und Z-Buffer lschen */

			glMatrixMode(GL_MODELVIEW); /* Projektionsmatrix beibehalten */

			glColor3f(1.0f, 1.0f, 1.0f);

			glPrint(260, 300, "Game Over", 0);
			sprintf(pre_string, "Points: %.0f", x);  /* Punkte anzeigen */
			glPrint(260, 280, pre_string, 0);

			SDL_GL_SwapBuffers();
	}

	set_default();
	scene_ende = GL_TRUE;
}

void draw_menu(void) /* Das Hauptmen anzeigen */
{	
	static GLfloat logo_angle = 0.0f; /* saves the current spin angle of the logo */
	static GLint menu_item = 1;	/* Speichert die aktuelle Menasuwahl */
	
	glClearColor(0.4f, 0.9f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); /* Farb- und Z-Buffer lschen */

	glMatrixMode(GL_MODELVIEW); /* Projektionsmatrix beibehalten */
	
	glLoadIdentity();
	
	glBindTexture(GL_TEXTURE_2D, texture[TEXTURE_menu]);
	glColor3f(1.0f, 1.0f, 1.0f);

	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(38.5f, -29.0f, -50.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(38.5f, 29.0f, -50.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-38.5f, 29.0f, -50.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-38.5f, -29.0f, -50.0f);
	glEnd();
	
	glDisable(GL_TEXTURE_2D);

    glEndList( );
	
	/* atom_logo begin */
		glTranslatef(-4.0f, -3.0f, -7.0f);
		glScalef(0.2f, 0.2f, 0.2f);
		/* the blender/gl-exporter does his job not very nice,
			so we disable culling*/
		glDisable(GL_CULL_FACE);
	
		glRotatef(logo_angle, 0.0f, 1.0f, 0.0f);	
		glCallList(DLIST_atom);
	
		if (logo_angle < 360.0f)
		{
			logo_angle += logo_spin * FrameInterval;
		}
		else
		{
			logo_angle = 0.0f;
		}
		
		/* Now we can reenable culling and texturing */
		glEnable(GL_CULL_FACE);
		glEnable(GL_TEXTURE_2D);
	/* atom_logo end*/
	
	/* Werden Tasten gedrckt? Wenn ja dann fhre die dazugehrige Aktion aus */
	if (key_up && menu_item > 1)
	{
		Mix_PlayChannel(-1, sound[SOUND_menuchange], 0);
		--menu_item; /* Im Men einen Eintrag nach oben */
		key_up = 0;
		
		if (menu_item == 2 && !level_created) --menu_item; /* if the 2. option isn't available */
	}

	if (key_down && menu_item < 3)
	{
		Mix_PlayChannel(-1, sound[SOUND_menuchange], 0);
		++menu_item; /* Im Men einen Eintrag nach unten */
		key_down = 0;
		
		if (menu_item == 2 && !level_created) ++menu_item; /* if the 2. option isn't available */
	}

	if (key_return)
	{
		key_return = !key_return;

		switch (menu_item)
		{
			case 1:
				set_default(); /* Szene zurcksetzen */
				create_level();
				levelrun = GL_TRUE;
				scene_ende = !scene_ende;
				break;
			case 2:
				set_default(); /* Szene zurcksetzen */
				levelrun = GL_TRUE;
				scene_ende = !scene_ende;
				break;
			case 3:
				quit(0);
				break;
			default:
				break;
		}
	}

	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, -6.0f); /* Kamera auf die richtige Stelle setzen */

	/* Mentext zeichnen: */
	glColor3f(1.0f, 1.0f, 1.0f);
	glPrint(200, 320, "Atom'o'run X-Mas Edition", 0);
	glPrint(200, 310, "------------------------", 0);

	/* 1. Eintrag */
	if (menu_item == 1)
	{
		glColor3f(0.5f, 1.0f, 0.0f);
	}
	else
	{
		glColor3f(0.1f, 0.5f, 0.0f);
	}
	glPrint(260, 295, "New Game", 0);

	/* 2. Eintrag */		
	if (menu_item == 2)
	{
		glColor3f(0.5f, 1.0f, 0.0f);
	}
	else
	{
		if (level_created) glColor3f(0.1f, 0.5f, 0.0f); else glColor3f(0.0f, 0.2f, 0.0f);
	}
	glPrint(260, 280, "Replay level", 0);
	
	/* 3. Eintrag */
	if (menu_item == 3)
	{
		glColor3f(0.5f, 1.0f, 0.0f);
	}
	else
	{
		glColor3f(0.1f, 0.5f, 0.0f);
	}
	glPrint(260, 260, "Exit", 0);
	
	glColor3f(1.0f, 1.0f, 1.0f);
	glPrint(430, 115, "Atom'o'run v"VERSION, 1);

	SDL_GL_SwapBuffers();
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
}

void menu(void) /* Die Hauptmenschleife */
{
	while(!scene_ende)
	{
		process_events(); /* Events abfragen */
		draw_menu(); /* Bild zeichnen */
		calcFrameRate(); /* FPS errechnen */
	}
	scene_ende = !scene_ende; /* Variable zurcksetzen */
}

void skip_intro()
{
	acidfogX = -15.5f;
	camx = -13.8f;
	intro_jump = GL_TRUE;
	lookleft = GL_FALSE;
	intro_running = GL_FALSE;
	stop_sound(SOUND_explode);
}

void draw_level(void) /* Das Level, etc zeichnen */
{
	static GLboolean key_up_old; /* avoid jumping around all the time, when the jumpkey is hold */
	static GLfloat charX;
	static GLfloat camX;
	static GLint soundleft;		/* control sound panning */
	static GLint soundright;	/* control sound panning */
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); /* Farb- und Z-Buffer lschen */

	glMatrixMode(GL_MODELVIEW); /* Projektionsmatrix beibehalten */

	/* Werden Tasten gedrckt? Wenn ja dann fhre die dazugehrige Aktion aus */
	/*
	if (key_up) camz+=cam_move*FrameInterval;
	if (key_down) camz-=cam_move*FrameInterval;
	if (key_right) camx-=cam_move*FrameInterval;
	if (key_left) camx+=cam_move*FrameInterval;
	*/
	
	if (intro_running) /* ignore user inputs, when the intro is running */
	{
		key_up = 0;
		key_right = 0;
		key_left = 0;
		key_right = 0;
	}
	
	if (!key_up)
	{
		key_up_old = 0;
	}
	
	if (key_up && !key_up_old)
	{		
		if (key_up && !jump && hoehe<7 && !jumptranslating)
		{
			Mix_PlayChannel(-1, sound[SOUND_jump], 0);
			key_up_old = 1;
			jump=1;
			xs=0;
			jumptranslating=1;
		}
	}
	glLoadIdentity();
	glCallList(DLIST_skybox); /* Die Skybox zeichnen */
	glTranslatef(camx,6,camz); /* Kamera auf die richtige Stelle setzen */
	
	/* Level zeichnen */
	for (loop=0; loop<LEVEL_SIZE_y; loop++)
	{
		for (loop2=0; loop2<LEVEL_SIZE_x; loop2++)
		{
			switch(lvl[loop2][loop])
			{
				case 'g':
					glCallList(DLIST_ground);
					glTranslatef(2.0f,0.0f,0.0f);
					break;
				case 'b':
					glCallList(DLIST_block);
					glTranslatef(2.0f,0.0f,0.0f);
					break;
				case 'a':
					glCallList(DLIST_acid);
					glTranslatef(2.0f,0.0f,0.0f);
					break;
				default:
					glTranslatef(2.0f,0.0f,0.0f);
					break;
			}
		}
		glTranslatef(LEVEL_SIZE_x * -2,-2.0f,0.0f); /* "Cursor" eine Zeile tiefer und nach links setzen */
	}
	glTranslatef(0,4,0);




	camx-=bombe*FrameInterval;	/* Level wird bei jedem Frame erhht. Dadurch scrollt das Level durch.	*/
	bombe+=0.1f*FrameInterval;	/* Das Level soll sich allerdings immer schneller nach rechts bewegen	*/

	/* ------------------------------------------ */

	/*	Wenn Der Charakter:
		1.	gerade springt
		2.	Der Springen-Button nicht gedrckt wird
		3.	Unten nicht frei ist
		4.	Er auf dem weg nach unten
		Praktisch: Wenn er nach dem springen wieder auf dem Boden steht!
		dann soll die Parabel auf den hchsten Punkt gesetzt werden */

	if (jumptranslating && !jump && !untenf && runter==1)
	{
		jumptranslating=0;
		xs=2;
	}

	if (untenf && !jumptranslating)	/*	Wenn unter dem Charakter frei ist, und er nicht gerade springt
										dann soll er langsam runter fallen (Parabel x wird erhht)*/
	{
		 xs+=5*FrameInterval;
	}

	y=-1*(xs-sprunghoehe)*(xs-sprunghoehe);	/*	Hier wird die aktuelle Hhe mit Hilfe einer Parabel berechnet.	*/

							/*	Wenn der Charakter springt, dann wird der button zum springen deaktiviert,
							und xs (der Parabel x wert) hochgezhlt. Dadurch gewinnt er an hoehe und verliert sie langsam wieder.
							Dazu muss die Matrix aber um die sprunghoehe*sprunghoehe Hoehe versetzt werden, damit der
							Charakter von dem Boden aus los springt.*/
	if (jumptranslating)
	{
		jump=0;
		xs+=5*FrameInterval;
		y+=sprunghoehe*sprunghoehe;
	}

	if (hoehe>=12)
	{
		char_dead();			/*	Wenn er bei einer hoeheren Hoehe als 12 ist, dann */
										/*	gehe raus zum Menue (verloren!) */
		return; /* dont continue drawing the level */
	}

	/*	Wenn der User eine der Move-Tasten bettigt, wird die Laufanimations Variable auf TRUE gesetzt, dadurch
		wird er dann spter im code die laufanimation starten!	*/
	if (key_right || key_left) moving = GL_TRUE; else moving = GL_FALSE;


	if (key_right==1 && key_left==0) lookleft=GL_FALSE;	/*	Wenn der Spieler nur die "nach-rechts" Taste drckt,
															guckt der Charakter nicht nach links	*/


	if (key_right==0 && key_left==1) lookleft=GL_TRUE;	/*	Wenn der Spieler nur die "nach-links" Taste drckt,
															guckt der Charakter nach links !	*/


	Position=x/2+1;	/*	Position in der hlfte der Opengl Koordinaten berechnen. Es werden	*/
	Position2=x/2;	/*	zwei stck. bentigt, da der Charakter ja zwischen 2 Position sein kann	*/

					/*	Die Position wird erst bei Hindernissen im Level gebraucht. Dies ist in dieser
						Verison noch nicht mglich	*/


	hoehe=6-(y-0.9);	/*	Das gleiche wie bei der Position gilt fr die hhe	*/
	hoehe2=6-(y+0.1);

	/* Im Nachfolgenden Code wird berprft, wo berall kein Hinterniss ist. In dieser
	Atomorun Version ist eigentlich nur interessant ob unten ein Hinderniss (Boden) ist oder nicht*/

	if(lvl[Position][hoehe]=='b')
	{
		rechtsf=0;
	} else rechtsf=1;

	if(lvl[Position2][hoehe]=='b')
	{
		linksf=0;
	} else linksf=1;

	if(((lvl[Position][hoehe+1]=='g' || lvl[Position][hoehe+1]=='b')) || lvl[Position2][hoehe+1]=='g' || lvl[Position2][hoehe+1]=='b')
	{
		untenf=0;
	} else untenf=1;


	if(lvl[Position][hoehe-1]=='g' || lvl[Position][hoehe-1]=='b' || lvl[Position2][hoehe-1]=='g' || lvl[Position2][hoehe-1]=='b')
	{
		obenf=0;
	} else obenf=1;




	if (xs>sprunghoehe) runter=1; else runter=0;	/*	Wenn xs groesser als sprunghoehe ist,
														dann ist der Charakter auf dem weg nach unten	*/

	if (camx-(-x-12) <=  0) /* beim berschreitem des linken Bildschirmrandes is Ende */
	{
		char_dead();
		return; /* dont continue drawing the level */
	}
	
	/* stereo effect for all character-sounds (using linear functions): */
	camX = camx;
	charX = x;
	
	soundleft = ((-10.625) * (camX-(-charX-12)) +255);
	if (soundleft < 0) soundleft = 0;
	soundright = ((10.625) * (camX-(-charX-12)));
	if (soundright > 255) soundright = 255;
		
	Mix_SetPanning(sound_channels[SOUND_walk], soundleft, soundright);
	Mix_SetPanning(sound_channels[SOUND_jump], soundleft, soundright);
	/* stereo end */
	
	glDisable(GL_TEXTURE_2D);	/*	Texturen deaktivieren, da der Charakter
									ohne Textur gezeichnet werden soll.*/
	glDisable(GL_CULL_FACE);	/*	Hintere Flchen verstecken muss deaktiviert werden.
									Dafr wurde der Charakter falsch exportiert	*/
	glEnable(GL_LIGHTING);		/*	Licht aktivieren. Dadurch wird der Charakter beleuchtet.	*/
	glColor4f(1.0,1.0,1.0,1.0);	/*	Farbe aus wei setzen. Alles was nun gezeichnet wird, wird wei (der Charakter also)	*/
	glTranslatef(x, y*1.2+0.2, 0);/*Matrix um x und y verschieben (y=berechnete hoehe, an der der Charakter hin soll) */
								  /*x steht fr die x achse auf der der Charakter gezeichnet wird. Durch Pfeiltasten verndert sich x */

	glScalef(0.2f, 0.2f, 0.2f);	/* Charakter muss verkleinert (scaliert) werden.	*/


	if (key_left==1 && moving)	/* Wenn der linke Pfeil gedrckt wird, dann soll x verkleinert werden.	*/
	{
		x-=13*FrameInterval;
	}

	if (key_right==1 && moving)/* Wenn der rechte Pfeil gedrckt wird, dann soll x vergrert werden.	*/
	{
		x+=13*FrameInterval;
	}

	if (lookleft) glScalef(-1,1,1);	/*	Sobald lookleft=1 ist (Charakter bewegt sich nach links), muss er gespiegelt werden.	*/

	if (moving)	/*	Wenn der Charakter sich bewegt soll der Oberkoerper und die Brille etwas gedreht gezeichnet werden.
					Wenn er sich nicht bewegt, wird er ohne Drehung gezeichnet.	*/
	{
		glRotatef(-20,0,0,1);
		DrawBody();
		DrawGlasses();
		glRotatef(20,0,0,1);
	}
	else
	{
		DrawBody();
		DrawGlasses();
	}

	glColor4f(1.0,1.0,1.0,1.0);	/*	Farbe wurde gendert, bevor die Brille gezichnet wurde. Wieder auf wei setzen ! */
	glTranslatef(-1.5f,-5.0f,1);/*	Matrix nach unten links verschieben, dort werden die Fe gezeichnet.	*/

	glPushMatrix();	/* Temporre Matrix erstellen, damit nciht die ganze Scene gedreht transformiert wird.	*/

	if (moving && !jumptranslating)	/*	Wenn der Charakter luft, und nicht in der Luft ist:	*/
	{
		
		play_sound(SOUND_walk);
		
		if (xrots2<=280.0f && xrots2>=260.0f)	/*	Sobald der Fuss bei einem Schritt vorne den Boden berhrt (das passiert wenn
													xrots2==270 ist), dann soll Der Fuss langsam nach hinten Transformiert werden	*/
		{
			xversatz1+=20.0f*FrameInterval;		/*	Fuss nach hinten Bewegen	*/
			glTranslatef(-xversatz1,0,0);

			if (xversatz1<=3 && xversatz1>=1.6)	/* sobald Der Fuss wieder nach hinten geschoben wurde: */
			{
				xversatz1=-1.6f;				/* Wird der Fuss an die Ausgangsposition gebracht */
				xrots2=90;						/* und die Drehung fngt von vorne an	*/
			}
		}
		else	/* Else bereich: Wenn der Fuss den Boden vorne nicht berhrt (Also wenn der Fuss sich in der Luft drehen soll) */
		{

		xrots2+=1000.0f*FrameInterval;	/* Dann nimmt die Drehung immer weiter zu	*/

		glRotatef(-xrots2,0,0,1);	/* Hier wird die Matrix fr den Fuss nun verschoben:	*/
		glTranslatef(0,-1.5f,0);
		glRotatef(xrots2,0,0,1);
		}
	}
	else
	{
		stop_sound(SOUND_walk);
	}

	DrawFoot();	/* Fuss wird in der speziell fr ihn ausgerichteten Matrix gezeichnet	*/
	glPopMatrix();

	glTranslatef(0,0,-3);	/* Matrix ein wenig nach hinten verschieben m den hinteren Fuss zu verschieben:	*/

							/* Hintere Fuss bewegt sich anch dem gleichen Prinzip wie der vordere Fuss	*/
	glPushMatrix();
	if (moving && !jumptranslating)
	{
		if (yrots2<=280.0f && yrots2>=260.0f)
		{
			yversatz1+=20.0f*FrameInterval;
			glTranslatef(-yversatz1,0,0);
			if (yversatz1<=3 && yversatz1>=1.6)
			{
				yversatz1=-1.6f;
				yrots2=90;
			}
		}
		else
		{
		if (yrots2>=360.0f) yrots2=xrots2-180;
		yrots2+=1000.0f*FrameInterval;
		glRotatef(-yrots2,0,0,1);
		glTranslatef(0,-1.5f,0);
		glRotatef(yrots2,0,0,1);
		}
	}
	DrawFoot();
	glPopMatrix();

	glEnable(GL_CULL_FACE);	/*	"versteckte Flchen ausblenden" wieder aktivieren,
								fr das Level zeichnen im nchsten Frame	*/
	glDisable(GL_LIGHTING);	/*	Licht deaktivieren, Level darf nicht beleuchtet werden (besitzt eine Textur)	*/

	glPopMatrix();


	/* ------------------------------------------ */

	glEnable(GL_TEXTURE_2D);
				
				
	glLoadIdentity();
	
	if (intro_running)
	{
		if (acidfogX < -15.5f)
		{
			if (!bomb_exploded) /* play needed sounds */
			{				
				sound_channels[SOUND_explode] = Mix_PlayChannel(-1, sound[SOUND_explode], 0); /* explode bomb */
				
				bomb_exploded = GL_TRUE;
			}
			
			acidfogX += acid_move * FrameInterval;
		}
		if (acidfogX > -15.8f && !intro_jump) /* the next block, lets Atomio jump once */
		{
			Mix_PlayChannel(-1, sound[SOUND_jump], 0);
			jump=1;
			xs=0;
			jumptranslating=1;
			
			intro_jump = GL_TRUE;
		}

		if (acidfogX > -15.5f)
		{
			skip_intro();
		}
	}
	
	if (intro_jump && camx > -21.0f)
		{
			glDisable(GL_TEXTURE_2D);
			glColor3f(0.5f, 0.5f, 0.5f);
			glBegin(GL_QUADS);
				glTexCoord2f(1.0f, 1.0f); glVertex3f(24.5f, 2.0f, -49.0f);
				glTexCoord2f(1.0f, 0.0f); glVertex3f(24.5f, 10.0f, -49.0f);
				glTexCoord2f(0.0f, 0.0f); glVertex3f(-23.5f, 10.0f, -49.0f);
				glTexCoord2f(0.0f, 1.0f); glVertex3f(-23.5f, 2.0f, -49.0f);
			glEnd();
			glEnable(GL_TEXTURE_2D);
			
			glColor3f(1.0f, 1.0f, 1.0f);
			//glPrint(140,250, "Oh my god, the bomb has just exploded!", 1);
			glPrint(140,300, "Oh my god, santa crashed with his", 1);
			glPrint(140,270, "nuclear driven sleigh. RUN AWAY!!!", 1);
			//glPrint(140,220, "RUN!!! You have to get outta here!", 1);
		}
	
	
	glTranslatef(acidfogX, 0.0f, camz);
	glCallList(DLIST_acidfog);
				
	sprintf(pre_string, "%dfps", fps);  /* FPS anzeigen */
	glColor3f(1.0f, 1.0f, 1.0f);
	glPrint(500, 460, pre_string, 0);

	sprintf(pre_string, "Points: %.0f", x);  /* Punkte anzeigen */
	glPrint(500, 440, pre_string, 0);

	SDL_GL_SwapBuffers();
}

void level(void) /* Die Levelschleife */
{
	play_sound(SOUND_wind);	/* the wind */
	
	/* the acid fog */
	play_sound(SOUND_atom);
	Mix_SetPanning(sound_channels[SOUND_atom], 255, 20); /* the fog is left from us, so we hear it from the left */
	
	if (Mix_PlayingMusic()) Mix_ResumeMusic();
			else Mix_PlayMusic(music, -1);
	
		while(!scene_ende)
		{
			process_events(); /* Events abfragen */
			draw_level(); /* Bild zeichnen */
			calcFrameRate(); /* FPS errechnen */
		}
		
	Mix_PauseMusic();
	
	stop_all_sounds();
	scene_ende = !scene_ende; /* Variable zurcksetzen */
}
